@@ -14,7 +14,9 @@
 #' are any combinations of parameters \code{"a"}, \code{"b"}, \code{"c"}, and \code{"d"}. Default value
 #' is \code{NULL}. See \strong{Details}.
 #' @param method character: method used to estimate parameters. The options are \code{"nls"} for
-#' non-linear least squares (default) and \code{"likelihood"} for maximum likelihood method.
+#' non-linear least squares (default), \code{"likelihood"} for maximum likelihood method with
+#' \code{"L-BFGS-B"} algorithm, or \code{"irls"} for maximum likelihood method with iteratively
+#' reweighted least squares (available only for \code{model = "2PL"}).
 #' @param match character or numeric: matching criterion to be used as estimate of trait. Can be
 #' either \code{"zscore"} (default, standardized total score), \code{"score"} (total test score),
 #' or numeric vector of the same length as number of observations in \code{Data}.
@@ -61,7 +63,7 @@
 #'
 #' The unconstrained form of 4PL generalized logistic regression model for probability of correct
 #' answer (i.e., \eqn{y = 1}) is
-#' \deqn{P(y = 1) = (c + cDif*g) + (d + dDif*g - c - cDif*g)/(1 + exp(-(a + aDif*g)*(x - b - bDif*g))), }
+#' \deqn{P(y = 1) = (c + cDif * g) + (d + dDif * g - c - cDif * g) / (1 + exp(-(a + aDif * g) * (x - b - bDif * g))), }
 #' where \eqn{x} is by default standardized total score (also called Z-score) and \eqn{g} is a group membership.
 #' Parameters \eqn{a}, \eqn{b}, \eqn{c}, and \eqn{d} are discrimination, difficulty, guessing, and inattention.
 #' Terms \eqn{aDif}, \eqn{bDif}, \eqn{cDif}, and \eqn{dDif} then represent differences between two groups
@@ -90,7 +92,7 @@
 #' (parameter \code{"a"}) and inattention (parameter \code{"d"}) are fixed for both groups and other parameters
 #' (\code{"b"} and \code{"c"}) are not. The \code{NA} value for \code{constraints} means no constraints.
 #'
-#' In case that model considers difference in guessing or inattention parameter, the different parameterization is
+#' In case that the model considers a difference in guessing or inattention parameter, different parameterization is
 #' used and parameters with standard errors are re-calculated by delta method.
 #'
 #' @return A list with the following arguments:
@@ -98,24 +100,24 @@
 #'   \item{\code{Sval}}{the values of \code{test} statistics.}
 #'   \item{\code{pval}}{the p-values by \code{test}.}
 #'   \item{\code{adjusted.pval}}{adjusted p-values by \code{p.adjust.method}.}
-#'   \item{\code{df}}{the degress of freedom of \code{test}.}
+#'   \item{\code{df}}{the degrees of freedom of \code{test}.}
 #'   \item{\code{test}}{used test.}
-#'   \item{\code{par.m0}}{the matrix of estimated item parameters for m0 model.}
-#'   \item{\code{se.m0}}{the matrix of standard errors of item parameters for m0 model.}
-#'   \item{\code{cov.m0}}{list of covariance matrices of item parameters for m0 model.}
-#'   \item{\code{par.m1}}{the matrix of estimated item parameters for m1 model.}
-#'   \item{\code{se.m1}}{the matrix of standard errors of item parameters for m1 model.}
-#'   \item{\code{cov.m1}}{list of covariance matrices of item parameters for m1 model.}
+#'   \item{\code{par.m0}}{the matrix of estimated item parameters for null model.}
+#'   \item{\code{se.m0}}{the matrix of standard errors of item parameters for null model.}
+#'   \item{\code{cov.m0}}{list of covariance matrices of item parameters for null model.}
+#'   \item{\code{par.m1}}{the matrix of estimated item parameters for alternative model.}
+#'   \item{\code{se.m1}}{the matrix of standard errors of item parameters for alternative model.}
+#'   \item{\code{cov.m1}}{list of covariance matrices of item parameters for alternative model.}
 #'   \item{\code{conv.fail}}{numeric: number of convergence issues.}
 #'   \item{\code{conv.fail.which}}{the indicators of the items which did not converge.}
-#'   \item{\code{ll.m0}}{log-likelihood of m0 model.}
-#'   \item{\code{ll.m1}}{log-likelihood of m1 model.}
+#'   \item{\code{ll.m0}}{log-likelihood of null model.}
+#'   \item{\code{ll.m1}}{log-likelihood of alternative model.}
 #'   \item{\code{startBo0}}{the binary matrix. Columns represents iterations of initial values
-#'   re-calculations, rows represents items. The value of 0 means no convergence issue in m0 model,
-#'   1 means convergence issue in m0 model.}
+#'   re-calculations, rows represents items. The value of 0 means no convergence issue in null model,
+#'   1 means convergence issue in null model.}
 #'   \item{\code{startBo1}}{the binary matrix. Columns represents iterations of initial values
-#'   re-calculations, rows represents items. The value of 0 means no convergence issue in m1 model,
-#'   1 means convergence issue in m1 model.}
+#'   re-calculations, rows represents items. The value of 0 means no convergence issue in alternative model,
+#'   1 means convergence issue in alternative model.}
 #' }
 #'
 #' @author
@@ -136,8 +138,11 @@
 #' A non-IRT approach accounting for guessing. Journal of Educational Measurement, 54(4), 498--517,
 #' \doi{10.1111/jedm.12158}.
 #'
+#' Hladka, A. (2021). Statistical models for detection of differential item functioning. Dissertation thesis.
+#' Faculty of Mathematics and Physics, Charles University.
+#'
 #' Hladka, A. & Martinkova, P. (2020). difNLR: Generalized logistic regression models for DIF and DDF detection.
-#' The R journal, 12(1), 300--323, \doi{10.32614/RJ-2020-014}.
+#' The R Journal, 12(1), 300--323, \doi{10.32614/RJ-2020-014}.
 #'
 #' Swaminathan, H. & Rogers, H. J. (1990). Detecting differential item functioning using logistic regression procedures.
 #' Journal of Educational Measurement, 27(4), 361--370, \doi{10.1111/j.1745-3984.1990.tb00754.x}
@@ -146,20 +151,19 @@
 #'
 #' @examples
 #' \dontrun{
-#' # loading data based on GMAT
+#' # loading data
 #' data(GMAT)
+#' Data <- GMAT[, 1:20] # items
+#' group <- GMAT[, "group"] # group membership variable
 #'
-#' Data <- GMAT[, 1:20]
-#' group <- GMAT[, "group"]
-#'
-#' # Testing both DIF effects using LR test (default)
+#' # testing both DIF effects using LR test (default)
 #' # and model with fixed guessing for both groups
 #' NLR(Data, group, model = "3PLcg")
 #'
-#' # Using F test
+#' # using F test
 #' NLR(Data, group, model = "3PLcg", test = "F")
 #'
-#' # Testing both DIF effects with Benjamini-Hochberg correction
+#' # testing both DIF effects with Benjamini-Hochberg correction
 #' NLR(Data, group, model = "3PLcg", p.adjust.method = "BH")
 #'
 #' # 4PL model with the same guessing and inattention
@@ -173,8 +177,11 @@
 #' # to test difference in b
 #' NLR(Data, group, model = "4PL", constraints = "ac", type = "b")
 #'
-#' # using maximum likelihood estimation method
+#' # using maximum likelihood estimation method with L-BFGS-B algorithm
 #' NLR(Data, group, model = "3PLcg", method = "likelihood")
+#'
+#' # using maximum likelihood estimation method with iteratively reweighted least squares algorithm
+#' NLR(Data, group, model = "2PL", method = "irls")
 #' }
 #'
 #' @keywords DIF
@@ -214,26 +221,35 @@ NLR <- function(Data, group, model, constraints = NULL, type = "all",
     type <- rep(type, m)
   }
 
-  parameterization <- ifelse(model %in% c(
-    "3PLc", "3PL", "3PLd", "4PLcgd", "4PLd", "4PLcdg",
-    "4PLc", "4PL"
-  ),
-  "alternative",
-  "classic"
-  )
-  if (missing(start)) {
-    start <- startNLR(Data, group, model, match = x, parameterization = parameterization)
+  if (method == "irls") {
+    parameterization <- rep("logistic", m)
   } else {
-    if (is.null(start)) {
+    parameterization <- ifelse(model %in% c(
+      "3PLc", "3PL", "3PLd", "4PLcgd", "4PLd", "4PLcdg",
+      "4PLc", "4PL"
+    ),
+    "alternative",
+    "classic"
+    )
+  }
+
+  if (method == "irls") {
+    start <- NULL
+  } else {
+    if (missing(start)) {
       start <- startNLR(Data, group, model, match = x, parameterization = parameterization)
     } else {
-      for (i in 1:m) {
-        if (parameterization[i] == "alternative") {
-          start[[i]]["cDif"] <- start[[i]]["c"] + start[[i]]["cDif"]
-          start[[i]]["dDif"] <- start[[i]]["d"] + start[[i]]["dDif"]
-          names(start[[i]]) <- c("a", "b", "cR", "dR", "aDif", "bDif", "cF", "dF")
-        } else {
-          names(start[[i]]) <- c("a", "b", "c", "d", "aDif", "bDif", "cDif", "dDif")
+      if (is.null(start)) {
+        start <- startNLR(Data, group, model, match = x, parameterization = parameterization)
+      } else {
+        for (i in 1:m) {
+          if (parameterization[i] == "alternative") {
+            start[[i]]["cDif"] <- start[[i]]["c"] + start[[i]]["cDif"]
+            start[[i]]["dDif"] <- start[[i]]["d"] + start[[i]]["dDif"]
+            names(start[[i]]) <- c("a", "b", "cR", "dR", "aDif", "bDif", "cF", "dF")
+          } else {
+            names(start[[i]]) <- c("a", "b", "c", "d", "aDif", "bDif", "cDif", "dDif")
+          }
         }
       }
     }
@@ -280,9 +296,8 @@ NLR <- function(Data, group, model, constraints = NULL, type = "all",
   conv.fail <- sum(cfM0, cfM1)
   conv.fail.which <- which(cfM0 | cfM1)
 
-  # using starting values for bootstraped samples
-
-  if (initboot) {
+  # using starting values for bootstrapped samples
+  if (initboot & conv.fail > 0) {
     startM0 <- startM1 <- start
     startBo0 <- rep(0, m)
     startBo1 <- rep(0, m)
@@ -291,7 +306,7 @@ NLR <- function(Data, group, model, constraints = NULL, type = "all",
     set.seed(42)
     for (i in 1:nrBo) {
       if (conv.fail > 0) {
-        samp <- sample(1:dim(Data)[1], size = dim(Data)[1], replace = T)
+        samp <- sample(1:dim(Data)[1], size = dim(Data)[1], replace = TRUE)
         startalt <- startNLR(Data[samp, ], group[samp], model,
           match = x,
           parameterization = parameterization
@@ -359,134 +374,89 @@ NLR <- function(Data, group, model, constraints = NULL, type = "all",
     message(paste("Convergence failure in item", conv.fail.which, "\n"))
   }
 
-  # likelihood
+  # log-likelihood
   ll.m0 <- ll.m1 <- rep(NA, m)
   ll.m0[which(!cfM0)] <- sapply(m0[which(!cfM0)], logLik)
   ll.m1[which(!cfM1)] <- sapply(m1[which(!cfM1)], logLik)
 
   # parameters
-  par.m1 <- se.m1 <- lapply(1:m, function(i) {
-    structure(rep(NA, length(M[[i]]$M1$parameters)),
-      names = M[[i]]$M1$parameters
-    )
-  })
   par.m0 <- se.m0 <- lapply(1:m, function(i) {
     structure(rep(NA, length(M[[i]]$M0$parameters)),
       names = M[[i]]$M0$parameters
     )
   })
-  par.m1[which(!cfM1)] <- lapply(m1[which(!cfM1)], coef)
+  par.m1 <- se.m1 <- lapply(1:m, function(i) {
+    structure(rep(NA, length(M[[i]]$M1$parameters)),
+      names = M[[i]]$M1$parameters
+    )
+  })
   par.m0[which(!cfM0)] <- lapply(m0[which(!cfM0)], coef)
+  par.m1[which(!cfM1)] <- lapply(m1[which(!cfM1)], coef)
 
   # covariance structure
   cov.m0 <- cov.m1 <- as.list(rep(NA, m))
-  cov.m1[which(!cfM1)] <- lapply(m1[which(!cfM1)], vcov, sandwich)
-  cov.m0[which(!cfM0)] <- lapply(m0[which(!cfM0)], vcov, sandwich)
-
-  cov.fail1 <- which(sapply(cov.m1, is.null))
+  if (method == "irls") {
+    cov.m0[which(!cfM0)] <- lapply(lapply(m0[which(!cfM0)], summary), vcov)
+    cov.m1[which(!cfM1)] <- lapply(lapply(m1[which(!cfM1)], summary), vcov)
+  } else {
+    cov.m0[which(!cfM0)] <- lapply(m0[which(!cfM0)], vcov, sandwich)
+    cov.m1[which(!cfM1)] <- lapply(m1[which(!cfM1)], vcov, sandwich)
+  }
   cov.fail0 <- which(sapply(cov.m0, is.null))
-  cov.fail <- sort(union(cov.fail1, cov.fail0))
+  cov.fail1 <- which(sapply(cov.m1, is.null))
+  cov.fail <- sort(union(cov.fail0, cov.fail1))
 
   if (length(cov.fail) > 0) {
     message(paste(
       "Covariance matrix cannot be computed for item",
       cov.fail,
       "\n"
-    )
-    )
+    ))
   }
   conv.m0 <- setdiff(1:m, unique(c(which(cfM0), which(sapply(cov.m0[which(!cfM0)], function(x) is.null(x))))))
   conv.m1 <- setdiff(1:m, unique(c(which(cfM1), which(sapply(cov.m1[which(!cfM1)], function(x) is.null(x))))))
-  # se
+  # standard errors
   se.m0[conv.m0] <- lapply(cov.m0[conv.m0], function(x) sqrt(diag(x)))
   se.m1[conv.m1] <- lapply(cov.m1[conv.m1], function(x) sqrt(diag(x)))
 
   # delta method
-  for (i in 1:m) {
-    if (parameterization[i] == "alternative") {
-      if (!(i %in% conv.m0)) {
-        names(par.m0[[i]]) <- names(se.m0[[i]]) <- gsub("cF", "cDif", names(par.m0[[i]]))
-        names(par.m0[[i]]) <- names(se.m0[[i]]) <- gsub("dF", "dDif", names(par.m0[[i]]))
-        names(par.m0[[i]]) <- names(se.m0[[i]]) <- gsub("cR", "c", names(par.m0[[i]]))
-        names(par.m0[[i]]) <- names(se.m0[[i]]) <- gsub("dR", "d", names(par.m0[[i]]))
-      } else {
-        nms.m0 <- which(c("a", "b", "cR", "dR", "aDif", "bDif", "cF", "dF") %in% names(par.m0[[i]]))
+  dm.m0 <- lapply(1:m, function(i) {
+    switch(parameterization[[i]],
+      "logistic" = .deltamethod.NLR.log2irt(
+        par = par.m0[[i]], cov = cov.m0[[i]],
+        conv = (i %in% conv.m0),
+        cov_fail = cfM0[i]
+      ),
+      "alternative" = .deltamethod.NLR.alt2irt(
+        par = par.m0[[i]], cov = cov.m0[[i]],
+        conv = (i %in% conv.m0),
+        cov_fail = cfM0[i]
+      ),
+      "classic" = list(par = par.m0[[i]], cov = cov.m0[[i]], se = se.m0[[i]])
+    )
+  })
+  par.m0 <- lapply(dm.m0, function(x) x$par)
+  cov.m0 <- lapply(dm.m0, function(x) x$cov)
+  se.m0 <- lapply(dm.m0, function(x) x$se)
 
-        par.tmp.m0 <- setNames(
-          rep(0, 8),
-          c("a", "b", "cR", "dR", "aDif", "bDif", "cF", "dF")
-        )
-        par.tmp.m0[nms.m0] <- par.m0[[i]]
-        par.m0[[i]] <- setNames(
-          c(par.tmp.m0[1:6], par.tmp.m0[7] - par.tmp.m0[3], par.tmp.m0[8] - par.tmp.m0[4]),
-          c("a", "b", "c", "d", "aDif", "bDif", "cDif", "dDif")
-        )[nms.m0]
-
-        if (!(i %in% cov.fail0)) {
-          cov.tmp.m0 <- matrix(
-            0,
-            ncol = 8, nrow = 8,
-            dimnames = list(
-              c("a", "b", "cR", "dR", "aDif", "bDif", "cF", "dF"),
-              c("a", "b", "cR", "dR", "aDif", "bDif", "cF", "dF")
-            )
-          )
-          cov.tmp.m0[nms.m0, nms.m0] <- cov.m0[[i]]
-          cov.m0[[i]] <- deltamethod(
-            list(~x1, ~x2, ~x3, ~x4, ~x5, ~x6, ~ x7 - x3, ~ x8 - x4),
-            par.tmp.m0,
-            cov.tmp.m0,
-            ses = FALSE
-          )[nms.m0, nms.m0]
-          colnames(cov.m0[[i]]) <- rownames(cov.m0[[i]]) <- c("a", "b", "c", "d", "aDif", "bDif", "cDif", "dDif")[nms.m0]
-          se.m0[[i]] <- sqrt(diag(cov.m0[[i]]))
-        }
-      }
-    }
-  }
-
-  for (i in 1:m) {
-    if (parameterization[i] == "alternative") {
-      if (!(i %in% conv.m1)) {
-        names(par.m1[[i]]) <- names(se.m1[[i]]) <- gsub("cF", "cDif", names(par.m1[[i]]))
-        names(par.m1[[i]]) <- names(se.m1[[i]]) <- gsub("dF", "dDif", names(par.m1[[i]]))
-        names(par.m1[[i]]) <- names(se.m1[[i]]) <- gsub("cR", "c", names(par.m1[[i]]))
-        names(par.m1[[i]]) <- names(se.m1[[i]]) <- gsub("dR", "d", names(par.m1[[i]]))
-      } else {
-        nms.m1 <- which(c("a", "b", "cR", "dR", "aDif", "bDif", "cF", "dF") %in% names(par.m1[[i]]))
-
-        par.tmp.m1 <- setNames(
-          rep(0, 8),
-          c("a", "b", "cR", "dR", "aDif", "bDif", "cF", "dF")
-        )
-        par.tmp.m1[nms.m1] <- par.m1[[i]]
-        par.m1[[i]] <- setNames(
-          c(par.tmp.m1[1:6], par.tmp.m1[7] - par.tmp.m1[3], par.tmp.m1[8] - par.tmp.m1[4]),
-          c("a", "b", "c", "d", "aDif", "bDif", "cDif", "dDif")
-        )[nms.m1]
-
-        if (!(i %in% cov.fail1)) {
-          cov.tmp.m1 <- matrix(
-            0,
-            ncol = 8, nrow = 8,
-            dimnames = list(
-              c("a", "b", "cR", "dR", "aDif", "bDif", "cF", "dF"),
-              c("a", "b", "cR", "dR", "aDif", "bDif", "cF", "dF")
-            )
-          )
-          cov.tmp.m1[nms.m1, nms.m1] <- cov.m1[[i]]
-          cov.m1[[i]] <- deltamethod(
-            list(~x1, ~x2, ~x3, ~x4, ~x5, ~x6, ~ x7 - x3, ~ x8 - x4),
-            par.tmp.m1,
-            cov.tmp.m1,
-            ses = FALSE
-          )[nms.m1, nms.m1]
-          colnames(cov.m1[[i]]) <- rownames(cov.m1[[i]]) <- c("a", "b", "c", "d", "aDif", "bDif", "cDif", "dDif")[nms.m1]
-          se.m1[[i]] <- sqrt(diag(cov.m1[[i]]))
-        }
-      }
-    }
-  }
+  dm.m1 <- lapply(1:m, function(i) {
+    switch(parameterization[[i]],
+      "logistic" = .deltamethod.NLR.log2irt(
+        par = par.m1[[i]], cov = cov.m1[[i]],
+        conv = (i %in% conv.m1),
+        cov_fail = cfM1[i]
+      ),
+      "alternative" = .deltamethod.NLR.alt2irt(
+        par = par.m1[[i]], cov = cov.m1[[i]],
+        conv = (i %in% conv.m1),
+        cov_fail = cfM1[i]
+      ),
+      "classic" = list(par = par.m1[[i]], cov = cov.m1[[i]], se = se.m1[[i]])
+    )
+  })
+  par.m1 <- lapply(dm.m1, function(x) x$par)
+  cov.m1 <- lapply(dm.m1, function(x) x$cov)
+  se.m1 <- lapply(dm.m1, function(x) x$se)
 
   names(par.m1) <- names(par.m0) <- names(se.m1) <- names(se.m0) <- names(cov.m0) <- names(cov.m1) <- colnames(Data)
 
@@ -553,7 +523,11 @@ NLR <- function(Data, group, model, constraints = NULL, type = "all",
   adjusted.pval <- p.adjust(pval, method = p.adjust.method)
 
   results <- list(
-    Sval = switch(test, "F" = Fval, "LR" = LRval, "W" = Wval),
+    Sval = switch(test,
+      "F" = Fval,
+      "LR" = LRval,
+      "W" = Wval
+    ),
     pval = pval, adjusted.pval = adjusted.pval,
     df = df, test = test,
     par.m0 = par.m0, se.m0 = se.m0, cov.m0 = cov.m0,
@@ -564,3 +538,160 @@ NLR <- function(Data, group, model, constraints = NULL, type = "all",
   )
   return(results)
 }
+
+.deltamethod.NLR.log2irt <- function(par, cov, conv, cov_fail) {
+  # only for 2PL model
+  if (conv) {
+    par_names <- which(c("(Intercept)", "x", "g", "x:g") %in% names(par))
+    par_tmp <- setNames(
+      rep(0, 4),
+      c("(Intercept)", "x", "g", "x:g")
+    )
+    par_tmp[par_names] <- par
+    par_new <- setNames(
+      c(
+        par_tmp[2],
+        -par_tmp[1] / par_tmp[2],
+        par_tmp[4],
+        (par_tmp[1] * par_tmp[4] - par_tmp[2] * par_tmp[3]) / (par_tmp[2] * (par_tmp[2] + par_tmp[4]))
+      ),
+      c("a", "b", "aDif", "bDif")
+    )[par_names]
+
+    if (cov_fail) {
+      cov_new <- se_new <- NULL
+    } else {
+      cov_tmp <- matrix(
+        0,
+        ncol = 4, nrow = 4,
+        dimnames = list(
+          c("(Intercept)", "x", "g", "x:g"),
+          c("(Intercept)", "x", "g", "x:g")
+        )
+      )
+      cov_tmp[par_names, par_names] <- cov
+      cov_new <- msm::deltamethod(
+        list(~x2, ~ -x1 / x2, ~x4, ~ (x1 * x4 - x2 * x3) / (x2 * (x2 + x4))),
+        par_tmp,
+        cov_tmp,
+        ses = FALSE
+      )[par_names, par_names]
+      colnames(cov_new) <- rownames(cov_new) <- c("a", "b", "aDif", "bDif")[par_names]
+      se_new <- sqrt(diag(cov_new))
+    }
+
+    return(list(par = par_new, cov = cov_new, se = se_new))
+  }
+}
+
+#' @noRd
+.deltamethod.NLR.irt2log <- function(par, cov, conv, cov_fail) {
+  if (conv) {
+    par_names <- c("a", "b", "c", "d", "aDif", "bDif", "cDif", "dDif") %in% names(par)
+    par_names_new <- as.logical(c(
+      -par_names[1] * par_names[2],
+      par_names[1],
+      par_names[3],
+      par_names[4],
+      -par_names[1] * par_names[6] - par_names[5] * par_names[2] - par_names[5] * par_names[6],
+      par_names[5],
+      par_names[7],
+      par_names[8]
+    ))
+    par_names <- which(par_names)
+    par_names_new <- which(par_names_new)
+    par_tmp <- setNames(
+      rep(0, 8),
+      c("a", "b", "c", "d", "aDif", "bDif", "cDif", "dDif")
+    )
+    par_tmp[par_names] <- par
+    par_new <- setNames(
+      c(
+        -par_tmp[1] * par_tmp[2],
+        par_tmp[1],
+        par_tmp[3],
+        par_tmp[4],
+        -par_tmp[1] * par_tmp[6] - par_tmp[5] * par_tmp[2] - par_tmp[5] * par_tmp[6],
+        par_tmp[5],
+        par_tmp[7],
+        par_tmp[8]
+      ),
+      c("(Intercept)", "x", "c", "d", "g", "x:g", "cDif", "dDif")
+    )[par_names_new]
+
+    if (cov_fail) {
+      cov_new <- se_new <- NULL
+    } else {
+      cov_tmp <- matrix(
+        0,
+        ncol = 8, nrow = 8,
+        dimnames = list(
+          c("a", "b", "c", "d", "aDif", "bDif", "cDif", "dDif"),
+          c("a", "b", "c", "d", "aDif", "bDif", "cDif", "dDif")
+        )
+      )
+      cov_tmp[par_names, par_names] <- cov
+      cov_new <- msm::deltamethod(
+        list(
+          ~ -x1 * x2, ~x1, ~x3, ~x4,
+          ~ -x1 * x6 - x5 * x2 - x5 * x6,
+          ~x5, ~x7, ~x8
+        ),
+        par_tmp,
+        cov_tmp,
+        ses = FALSE
+      )[par_names_new, par_names_new]
+      colnames(cov_new) <- rownames(cov_new) <- c("(Intercept)", "x", "c", "d", "g", "x:g", "cDif", "dDif")[par_names]
+      se_new <- sqrt(diag(cov_new))
+    }
+
+    return(list(par = par_new, cov = cov_new, se = se_new))
+  }
+}
+
+#' @noRd
+.deltamethod.NLR.alt2irt <- function(par, cov, conv, cov_fail) {
+  if (conv) {
+    par_names <- which(c("a", "b", "cR", "dR", "aDif", "bDif", "cF", "dF") %in% names(par))
+    par_tmp <- setNames(
+      rep(0, 8),
+      c("a", "b", "cR", "dR", "aDif", "bDif", "cF", "dF")
+    )
+    par_tmp[par_names] <- par
+    par_new <- setNames(
+      c(par_tmp[1:6], par_tmp[7] - par_tmp[3], par_tmp[8] - par_tmp[4]),
+      c("a", "b", "c", "d", "aDif", "bDif", "cDif", "dDif")
+    )[par_names]
+
+    if (cov_fail) {
+      cov_new <- se_new <- NULL
+    } else {
+      cov_tmp <- matrix(
+        0,
+        ncol = 8, nrow = 8,
+        dimnames = list(
+          c("a", "b", "cR", "dR", "aDif", "bDif", "cF", "dF"),
+          c("a", "b", "cR", "dR", "aDif", "bDif", "cF", "dF")
+        )
+      )
+      cov_tmp[par_names, par_names] <- cov
+      cov_new <- msm::deltamethod(
+        list(~x1, ~x2, ~x3, ~x4, ~x5, ~x6, ~ x7 - x3, ~ x8 - x4),
+        par_tmp,
+        cov_tmp,
+        ses = FALSE
+      )[par_names, par_names]
+      colnames(cov_new) <- rownames(cov_new) <- c("a", "b", "c", "d", "aDif", "bDif", "cDif", "dDif")[par_names]
+      se_new <- sqrt(diag(cov_new))
+    }
+  } else {
+    par_new <- par
+    se_new <- sqrt(diag(cov))
+    names(par_new) <- names(se_new) <- gsub("cF", "cDif", names(par))
+    names(par_new) <- names(se_new) <- gsub("dF", "dDif", names(par))
+    names(par_new) <- names(se_new) <- gsub("cR", "c", names(par))
+    names(par_new) <- names(se_new) <- gsub("dR", "d", names(par))
+  }
+
+  return(list(par = par_new, cov = cov_new, se = se_new))
+}
